.criteria { 
      margin-bottom: 10px;
 }

.criteria-group-title 
{
  font-weight: bold;
  background-color: #E0E0E0;
  padding: 5px;
  margin-bottom: 10px;
 }
.mapcanvas img { max-width: none; }

.date { width: 100px; }
.time { width: 100px; }
.duration { width: 100px; }
.filterexpr { width: 300px; }

input[type="file"] {
    padding-top: 5px;
    line-height: 0;
}

